@@ -2,52 +2,56 @@
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details.
 
-   $Id: opennap.h,v 1.220 2000/08/13 22:57:09 drscholl Exp $ */
+   $Id: opennap.h,v 1.293 2001/03/03 01:00:06 drscholl Exp $ */
 
 #ifndef opennap_h
 #define opennap_h
 
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
 #include <windows.h>
 /* the next two #defines are needed for zlib */
 #define _WINDOWS
 #define ZLIB_DLL
 #include "zlib.h"
 #endif
+
+#if HAVE_POLL
+#include <sys/poll.h>
+#endif
+
 #include <stdarg.h>
 #include <sys/types.h>
 #include <zlib.h>
 #include "hash.h"
 #include "list.h"
 
-#define OUTOFMEMORY(f) log("%s(): OUT OF MEMORY at %s:%d", f, __FILE__, __LINE__)
-#define _logerr(f,s,e) log("%s(): %s: %s (errno %d)", f, s, strerror(e), e)
+#define OUTOFMEMORY(f) log("%s: OUT OF MEMORY at %s:%d", f, __FILE__, __LINE__)
+#define _logerr(f,s,e) log("%s: %s: %s (errno %d)", f, s, strerror(e), e)
 #define logerr(f,s) _logerr(f,s,errno)
 
 #define MAGIC_USER 0xeaee402a
 #define MAGIC_CHANNEL 0xa66544cb
-#define MAGIC_HOTLIST 0xb0f8ad23
 #define MAGIC_CONNECTION 0x3c4474a3
 #define MAGIC_BUFFER 0xe5a7a3be
 #define MAGIC_CHANUSER 0x728dc736
 #define MAGIC_OPS 0xa28e453f
+#define MAGIC_FLIST 0xff496a95
 
 #if WORDS_BIGENDIAN
 /* convert the bytes of a 16-bit integer to little endian */
-#define BSWAP16(c) (((c & 0xff) << 8) | ((c >> 8) & 0xff))
+#define BSWAP16(c) (((((c) & 0xff) << 8) | (((c) >> 8) & 0xff)))
 /* convert the bytes of a 32-bit integer to little endian */
-#define BSWAP32(c) ((c>>24)&0xff)|((c>>8)&0xff00)|((c<<8)&0xff0000)|(c<<24)
+#define BSWAP32(c) ((((c)>>24)&0xff)|(((c)>>8)&0xff00)|(((c)<<8)&0xff0000)|((c)<<24))
 #else
 #define BSWAP16(c) c
 #define BSWAP32(c) c
 #endif
 
-#define ISSPACE(c) isspace((unsigned char)c)
-#define ISPRINT(c) isprint((unsigned char)c)
+#define ISSPACE(c) isspace((unsigned char)(c))
+#define ISPRINT(c) isprint((unsigned char)(c))
 
 #define FIELDS(p) sizeof(p)/sizeof(char*)
 
-typedef unsigned char uchar;
 
 typedef struct _buffer BUFFER;
 
@@ -58,57 +62,58 @@ struct _buffer
 #if DEBUG
     unsigned int magic;
 #endif
-    char *data;			/* allocated data */
-    int datasize;		/* total bytes used in `data' */
-    int datamax;		/* size of allocated memory block */
-    int consumed;		/* how many bytes of data consumed from this buffer */
+    char   *data;		/* allocated data */
+    int     datasize;		/* total bytes used in `data' */
+    int     datamax;		/* size of allocated memory block */
+    int     consumed;		/* how many bytes of data consumed from this buffer */
     BUFFER *next;
 };
 
-#define BUFFER_SIZE 2048	/* default buffer length for output queues */
+#define BUFFER_SIZE 4096	/* default buffer length for output queues */
 
 typedef struct _connection CONNECTION;
 typedef struct _user USER;
 typedef struct _channel CHANNEL;
-typedef struct _hotlist HOTLIST;
 typedef struct _chanuser CHANUSER;
+typedef struct _class CLASS;
 
 /* bitmasks for the `flags' member of struct _chanuser */
-#define ON_OPERATOR	1
+#define ON_CHANNEL_OPERATOR	1
 #define ON_CHANNEL_VOICE	(1<<1)
 #define ON_CHANNEL_MUZZLED	(1<<2)
 
 struct _chanuser
 {
 #ifdef DEBUG
-    int magic;
+    unsigned int magic;
 #endif
-    int flags;
-    USER *user;
+    int     flags;
+    USER   *user;
 };
 
 /* bitmask values for the channel flags */
-#define ON_CHANNEL_USER		(1<<0)	/* user created channel */
+#define ON_CHANNEL_REGISTERED	(1<<0)	/* channel is registered */
 #define ON_CHANNEL_PRIVATE	(1<<1)	/* private (hidden) channel */
 #define ON_CHANNEL_MODERATED	(1<<2)	/* moderated channel */
 #define ON_CHANNEL_INVITE	(1<<3)	/* invitation required */
 #define ON_CHANNEL_TOPIC	(1<<4)	/* topic can be set by any member */
+#define ON_CHANNEL_QUIET	(1<<5)	/* quiet channel */
 
 struct _channel
 {
 #ifdef DEBUG
     unsigned int magic;
 #endif
-    char *name;			/* name of the channel */
-    char *topic;		/* current topic of discussion */
-    LIST *ops;			/* list of operators for the channel */
-    LIST *users;		/* list of users on the channel */
+    char   *name;		/* name of the channel */
+    char   *topic;		/* current topic of discussion */
+    LIST   *users;		/* list of users on the channel */
+    LIST   *bans;		/* channel specific bans */
+    LIST   *invited;		/* invited users */
+    time_t  timestamp;		/* when channel was created, used to sync servers */
     unsigned short limit;	/* max number of users allowed */
-    unsigned int flags:13;	/* channel flags */
+    unsigned int flags:12;	/* channel flags */
     unsigned int level:3;	/* minimum level to enter channel */
-    LIST *bans;			/* channel specific bans */
-    time_t timestamp;		/* used to sync servers */
-    LIST *invited;		/* invited users */
+    unsigned int local:1;	/* channel is local to server */
 };
 
 /* user level */
@@ -121,15 +126,25 @@ enum
     LEVEL_ELITE
 };
 
+/* connection class */
+struct _class
+{
+    unsigned int target;
+    unsigned int mask;
+    int     limit;
+};
+
 struct _user
 {
 #ifdef DEBUG
     unsigned int magic;
 #endif
-    char *nick;
-    char *pass;			/* password for this user, needed for sync */
-    char *clientinfo;
-    char *server;		/* which server the user is connected to.
+    char   *nick;
+    char   *pass;		/* password for this user, needed for sync */
+    char   *clientinfo;		/* DO NOT FREE.  This is a ref to
+				   an entry in the Client_Versions hash
+				   table */
+    char   *server;		/* which server the user is connected to.
 				   NOTE. this is a pointer to an entry in
 				   Server_Names, and not malloc'd. */
 
@@ -154,11 +169,10 @@ struct _user
 
     unsigned short port;	/* data port client is listening on */
     unsigned short conport;	/* remote port for connection to server */
-    time_t connected;		/* time at which the user connected */
-    LIST *channels;		/* channels of which this user is a member */
+    time_t  connected;		/* time at which the user connected */
+    LIST   *channels;		/* channels of which this user is a member */
     CONNECTION *con;		/* local connection, or server which this
 				   user is behind */
-    LIST *invited;		/* invited channels */
 };
 
 enum
@@ -182,8 +196,8 @@ SERVER;
 
 typedef struct
 {
-    char *nonce;
-    char *sendernonce;
+    char   *nonce;
+    char   *sendernonce;
 }
 AUTH;
 
@@ -198,9 +212,10 @@ typedef struct
        a pointer to the _single_ global entry stored in the Hotlist
        hash table.  the actual HOTLIST* pointer should only be freed
        when hotlist->numusers is zero.  */
-    LIST *hotlist;
-    HASH *files;		/* db entries for this user's shared files */
-    LIST *ignore;		/* server side ignore list */
+    LIST   *hotlist;
+    HASH   *files;		/* db entries for this user's shared files */
+    LIST   *ignore;		/* server side ignore list */
+    int     searches;		/* number of pending searches */
 }
 USEROPT;
 
@@ -209,13 +224,11 @@ struct _connection
 #ifdef DEBUG
     unsigned int magic;
 #endif
-    short id;			/* offset into the Client[] arrary for this
-				   instance */
-    unsigned short port;	/* remote port */
-    int fd;			/* socket for this connection */
+    int     id;			/* index into the Clients[] and Poll[] array */
+    int     fd;			/* socket for this connection */
     unsigned int ip;		/* ip for this connection */
-    char *host;			/* host from which this connection originates */
-    USER *user;			/* pointer to the user associated with this
+    char   *host;		/* host from which this connection originates */
+    USER   *user;		/* pointer to the user associated with this
 				   connection, if CLASS_USER */
     BUFFER *sendbuf;		/* output buffer */
     BUFFER *recvbuf;		/* input buffer */
@@ -231,12 +244,17 @@ struct _connection
 #define sopt opt.server
 	SERVER *server;
 	/* this field is used for the authentication phase of server links */
-	AUTH *auth;
+	AUTH   *auth;
     }
     opt;
 
-    time_t	timer;		/* timer to detect idle connections */
-    time_t	flood;		/* flood protection counter */
+    time_t  timer;		/* timer to detect idle connections */
+    time_t  flood_start;	/* flood protection counter */
+
+    unsigned short flood_commands;
+    unsigned short port;	/* remote port */
+
+    unsigned short unused;
 
     unsigned int connecting:1;
     unsigned int destroy:1;	/* connection should be destoyed in
@@ -247,31 +265,24 @@ struct _connection
 				   when it is safe */
     unsigned int killed:1;	/* set when the user was killed so free_user()
 				   knows not to generate a QUIT message */
+    unsigned int quit:1;	/* used to detect manual server disconnects */
     unsigned int server_login:1;
     unsigned int compress:4;	/* compression level for this connection */
     unsigned int class:2;	/* connection class (unknown, user, server) */
     unsigned int numerics:1;	/* use real numerics for opennap extensions */
-    unsigned int xxx:5;		/* unused */
+    unsigned int xxx:4;		/*unused */
 
-    short yyy; /* unused - remaining 16 bits of above bitmasks */
-};
-
-/* hotlist entry */
-struct _hotlist
-{
-#ifdef DEBUG
-    unsigned int magic;
-#endif
-    char *nick;			/* user being monitored */
-    LIST *users;
 };
 
 /* list of DATUM entries, used in the global file list */
 typedef struct
 {
-    char *key;			/* keyword */
-    LIST *list;			/* list of files containing this keyword */
-    int count;			/* number of files in the list */
+#if DEBUG
+    unsigned int magic;
+#endif
+    char   *key;		/* keyword */
+    LIST   *list;		/* list of files containing this keyword */
+    int     count;		/* number of files in the list */
 }
 FLIST;
 
@@ -289,59 +300,104 @@ enum
 
 typedef struct
 {
-    USER *user;			/* user who possesses this file */
-    char *filename;
+    USER   *user;		/* user who possesses this file */
+    char   *filename;
 #if RESUME
-    char *hash;			/* the md5 hash of the file */
+    char   *hash;		/* the md5 hash of the file */
 #endif
+    LIST   *tokens;		/* list of keywords for this file */
     unsigned int size;		/* size of file in bytes.  set to -1 if
 				   the structure is invalid */
     unsigned short duration;
     /* next 4 fields make up 16 bits */
-    unsigned int bitrate : 5;	/* offset into BitRate[] */
+    unsigned int bitrate:5;	/* offset into BitRate[] */
     unsigned int frequency:3;	/* offset into SampleRate[] */
     unsigned int type:3;	/* content type */
-    unsigned int refcount:5;	/* how many references to this structure? */
+    unsigned int xxx:5;		/* unused */
 }
 DATUM;
 
 typedef struct _ban
 {
-    char *target;	/* target of the ban */
-    char *setby;	/* user/server that set the ban */
-    char *reason;	/* reason for the ban */
-    time_t when;	/* when the ban was set */
-    int timeout;	/* how many secs the ban is active for */
+    char   *target;		/* target of the ban */
+    char   *setby;		/* user/server that set the ban */
+    char   *reason;		/* reason for the ban */
+    time_t  when;		/* when the ban was set */
+    int     timeout;		/* how many secs the ban is active for */
 }
 BAN;
 
 /* bitmask for use in USERDB .flags member */
 #define ON_MUZZLED 0x0001
-#define ON_CLOAKED 0x0002
 
 /* 20 (+4) bytes */
 typedef struct
 {
-    char *nick;
-    char *password;
+    char   *nick;
+    char   *password;
 #if EMAIL
-    char *email;
+    char   *email;
 #endif
     unsigned short level;
     unsigned short flags;
-    time_t timestamp;	/* when the record was last modified */
-    time_t lastSeen;
+    time_t  created;		/* when the record was created */
+    time_t  lastSeen;
 }
 USERDB;
 
 typedef struct link
 {
-    char *server;
-    char *peer;
+    char   *server;
+    char   *peer;
     unsigned short port;
     unsigned short peerport;
-    int hops;
-} LINK;
+    int     hops;
+}
+LINK;
+
+/* structure to hold the server-link authentications */
+
+typedef struct
+{
+    char   *name;
+    char   *alias;		/* name which will be displayed (optional) */
+    int     port;		/* port to connect to */
+    char   *their_pass;
+    char   *my_pass;
+}
+server_auth_t;
+
+/* who-was entry */
+typedef struct
+{
+    char   *nick;
+    char   *server;
+    unsigned int ip;		/* client's ip address */
+    time_t  when;		/* when the user logged out */
+    char   *clientinfo;		/* client version string */
+}
+whowas_t;
+
+/* information per ip address to help curb abuse of reconnecting to fast
+ * or registering a lot of nicknames.
+ */
+typedef struct
+{
+    unsigned int ip;		/* ip address */
+    time_t  last_connect;	/* time at which last connect occurred */
+    time_t  last_register;	/* time at which last was registered */
+    unsigned int users;		/* current number of users from this ip */
+    unsigned int connects;	/* total number of connects from this ip */
+}
+ip_info_t;
+
+typedef struct
+{
+    unsigned int ip;
+    unsigned int mask;
+    unsigned count;
+}
+access_t;
 
 /* bitmask used for new_tcp_socket() call */
 #define ON_NONBLOCKING	1
@@ -350,51 +406,116 @@ typedef struct link
 
 typedef void (*timer_cb_t) (void *);
 
-extern unsigned int Bytes_In;
-extern unsigned int Bytes_Out;
-extern int Channel_Limit;
+/* define a namespace for global variables */
+typedef struct
+{
+#if HAVE_POLL
+    struct pollfd *poll;
+    int     poll_num;
+    int     poll_max;
+    int    *fdmap;
+#else
+    fd_set  read_fds;
+    fd_set  write_fds;
+#endif
+    int     max_fd;
+
+    CONNECTION **clients;
+    int     clients_num;
+    int     clients_max;
+
+    double  bytes_in;
+    double  bytes_out;
+    double  total_bytes_in;
+    double  total_bytes_out;
+
+    time_t  current_time;
+    time_t  last_click;
+    int     min_read;
+    int     stat_click;
+    int     search_count;
+#ifndef ROUTING_ONLY
+    int     stat_server_fd;
+    char   *stat_pass;
+    char   *stat_server;
+    int     stat_server_port;
+    char   *stat_user;
+    char   *report_ip;
+    int     report_port;
+    char   *report_name;
+#endif
+}
+global_t;
+
+extern global_t global;
+
+#ifndef ROUTING_ONLY
+extern int File_Count_Threshold;
+extern HASH *File_Table;
+extern HASH *Filter;
+extern int Local_Files;
+extern int Max_Search_Results;
+extern int Max_Shared;
+
+#if RESUME
+extern HASH *MD5;
+#endif
+extern int Stats_Port;
+#endif
+
+extern HASH *Channel_Db;
 extern int Client_Queue_Length;
-extern int Collect_Interval;
+extern HASH *Client_Versions;
+extern HASH *Clones;
 extern int Compression_Level;
 extern char *Config_Dir;
-extern time_t Current_Time;
+extern unsigned int Connection_Count;
 extern int Flood_Commands;
 extern int Flood_Time;
+extern LIST *Flooders;
 extern unsigned int Interface;
-extern time_t Last_Click;
 extern char *Listen_Addr;
-extern int Local_Files;
+extern int Login_Interval;
 extern int Login_Timeout;
 extern int Max_Browse_Result;
 extern int Max_Command_Length;
 extern int Max_Connections;
 extern int Max_Nick_Length;
-extern int Max_Search_Results;
-extern int Max_Shared;
+extern int Max_Searches;
+extern int Max_Time_Delta;
 extern int Max_User_Channels;	/* # of channels is a user allowed to join */
+extern LIST *Mods;
 extern int Nick_Expire;
-extern unsigned int Search_Count;	/* # of searches in the last click */
+extern int Ping_Interval;
+extern int Register_Interval;
+extern char *Server_Alias;
 extern int Search_Timeout;
+extern int Server_Chunk;
 extern unsigned int Server_Flags;
+extern unsigned int Server_Ip;
 extern char *Server_Name;
 extern LIST *Server_Names;
-extern char *Server_Pass;
 extern LIST *Server_Ports;
+extern LIST *Server_Auth;	/* link auth info */
 extern int Server_Queue_Length;
 extern int SigCaught;		/* flag to control main loop */
-extern int Stat_Click;
-extern int Stats_Port;
 extern time_t Server_Start;
-extern unsigned int Total_Bytes_In;
-extern unsigned int Total_Bytes_Out;
 extern int User_Db_Interval;
 extern int Max_Channel_Length;
+extern int Max_Clones;
 extern int Max_Ignore;
 extern int Max_Hotlist;
 extern int Max_Topic;
 extern int Max_Client_String;
 extern int Max_Reason;
-extern int Max_Clones;
+extern int Num_Files;		/* total number of available files */
+extern double Num_Gigs;		/* total size of files available (in kB) */
+extern unsigned int Pending_Searches;
+extern int Warn_Time_Delta;
+extern HASH *Who_Was;
+extern int Who_Was_Time;
+extern unsigned int UserMode_int;	/* Default UserMode to use when mod logs in */
+extern char *UserMode;
 
 extern const int BitRate[];
 extern const int SampleRate[];
@@ -416,36 +537,32 @@ extern int Max_Rss_Size;
 #define ON_NO_LISTEN		(1<<4)	/* don't listen on port 8889 */
 #define ON_BACKGROUND		(1<<5)	/* run in daemon mode */
 #define ON_EJECT_WHEN_FULL	(1<<6)	/* eject nonsharing clients when full */
+#define ON_IRC_CHANNELS		(1<<7)	/* irc-style channels */
+#define ON_LOGLEVEL_CHANGE	(1<<8)	/* log level changes to clients */
+#define ON_REMOTE_BROWSE	(1<<9)	/* allow server->server browsing */
+#define ON_AUTO_LINK		(1<<10)	/* auto-link servers */
+#define ON_GHOST_KILL		(1<<11)	/* enable auto ghost kill */
+#define ON_ALLOW_SHARE		(1<<12)	/* allow file sharing */
+#define ON_RESTRICT_REGISTRATION	(1<<13)	/* don't allow auto nick reg. */
 
 extern char Buf[2048];
 
-extern CONNECTION **Clients;	/* locally connected clients */
-extern int Num_Clients;
-extern int Max_Clients;
-
-extern int Num_Files;		/* total number of available files */
-extern double Num_Gigs;		/* total size of files available (in kB) */
-
 extern LIST *Bans;
 extern LIST *Servers;		/* peer servers */
 extern LIST *Server_Links;
+extern LIST *UserClass;
 
 extern HASH *Users;
 extern HASH *Channels;
 extern HASH *Hotlist;
-extern HASH *File_Table;
-#if RESUME
-extern HASH *MD5;
-#endif
 extern HASH *User_Db;
-extern HASH *Filter;
 
 extern char *Levels[LEVEL_ELITE + 1];
 extern char *Content_Types[CT_UNKNOWN];
 
 #define set_tag(b,n) set_val(b+2,n)
 #define set_len set_val
-void set_val (char *d, unsigned short val);
+void    set_val (char *d, unsigned short val);
 
 /* message types */
 /* MSG_CLIENT_* are messages sent by the client to the server
@@ -457,6 +574,7 @@ void set_val (char *d, unsigned short val);
 #define MSG_SERVER_ERROR		0
 #define MSG_CLIENT_LOGIN		2
 #define MSG_SERVER_EMAIL		3
+#define MSG_CLIENT_VERSION_CHECK	4
 #define MSG_CLIENT_LOGIN_REGISTER	6
 #define MSG_CLIENT_REGISTER		7
 #define MSG_SERVER_REGISTER_OK		8
@@ -465,8 +583,10 @@ void set_val (char *d, unsigned short val);
 #define MSG_CLIENT_CHECK_PASS		11
 #define MSG_SERVER_PASS_OK		12
 #define MSG_SERVER_ECHO			13
+#define MSG_CLIENT_REGISTRATION_INFO	14
 #define MSG_CLIENT_ADD_FILE		100
 #define MSG_CLIENT_REMOVE_FILE		102
+#define MSG_CLIENT_UNSHARE_ALL		110
 #define MSG_CLIENT_SEARCH		200
 #define MSG_SERVER_SEARCH_RESULT	201
 #define MSG_SERVER_SEARCH_END		202
@@ -491,6 +611,7 @@ void set_val (char *d, unsigned short val);
 #define MSG_CLIENT_UPLOAD_END		221
 #define MSG_CLIENT_CHECK_PORT		300
 #define MSG_SERVER_HOTLIST_ACK		301
+#define MSG_SERVER_HOTLIST_ERROR	302
 #define MSG_CLIENT_REMOVE_HOTLIST	303
 #define MSG_SERVER_DISCONNECTING	316
 #define MSG_CLIENT_IGNORE_LIST		320
@@ -551,6 +672,9 @@ void set_val (char *d, unsigned short val);
 #define MSG_CLIENT_ANNOUNCE		628
 #define MSG_SERVER_ANNOUNCE		628	/* same as client message */
 #define MSG_SERVER_NICK_BANLIST		629
+#define MSG_CLIENT_BROWSE_DIRECT	640
+#define MSG_SERVER_BROWSE_DIRECT_OK	641
+#define MSG_SERVER_BROWSE_DIRECT_ERR	642
 #define MSG_CLIENT_CLOAK		652
 #define MSG_CLIENT_CHANGE_SPEED		700
 #define MSG_CLIENT_CHANGE_PASS		701
@@ -587,7 +711,7 @@ void set_val (char *d, unsigned short val);
 #define MSG_SERVER_LOGIN		10010	/* server login request */
 #define MSG_SERVER_LOGIN_ACK		10011	/* server login response */
 #define MSG_SERVER_USER_SHARING		10012
-#define MSG_SERVER_USER_IP		10013	/* ip for user */
+#define MSG_SERVER_USER_IP		10013	/* deprecated, ip for user */
 #define MSG_SERVER_REGINFO		10014	/* registration info */
 #define MSG_SERVER_REMOTE_SEARCH	10015
 #define MSG_SERVER_REMOTE_SEARCH_RESULT	10016
@@ -596,6 +720,9 @@ void set_val (char *d, unsigned short val);
 #define MSG_SERVER_LINK_INFO		10019
 #define MSG_SERVER_QUIT			10020
 #define MSG_SERVER_NOTIFY_MODS		10021
+#define MSG_SERVER_SERVER_PONG		10022	/* server->server pong */
+#define MSG_SERVER_TIME_CHECK		10023	/* server time check */
+#define MSG_SERVER_WHOIS_NOTIFY		10024
 #define MSG_CLIENT_CONNECT		10100
 #define MSG_CLIENT_DISCONNECT		10101
 #define MSG_CLIENT_KILL_SERVER		10110
@@ -605,9 +732,16 @@ void set_val (char *d, unsigned short val);
 #define MSG_CLIENT_USAGE_STATS		10115	/* server usage stats */
 #define MSG_SERVER_USAGE_STATS		10115
 #define MSG_CLIENT_REHASH		10117	/* reload config file */
+#define MSG_CLIENT_VERSION_STATS	10118	/* client version stats */
+#define MSG_CLIENT_WHICH_SERVER		10119	/* which server a user is on */
+#define MSG_CLIENT_PING_ALL_SERVERS	10120
+#define MSG_CLIENT_WHO_WAS		10121
+#define MSG_CLIENT_MASS_KILL		10122
+#define MSG_CLIENT_HISTOGRAM		10123
+#define MSG_SERVER_HISTOGRAM		10124
 #define MSG_CLIENT_REGISTER_USER	10200
-#define MSG_CLIENT_CHANNEL_LEVEL	10201	/* deprecated, use 823 instead*/
-#define MSG_CLIENT_KICK_USER		10202	/* deprecated, use 829 instead*/
+#define MSG_CLIENT_CHANNEL_LEVEL	10201	/* deprecated, use 823 instead */
+#define MSG_CLIENT_KICK_USER		10202	/* deprecated, use 829 instead */
 #define MSG_CLIENT_USER_MODE		10203	/* set a user mode */
 #define MSG_SERVER_USER_MODE		10203
 #define MSG_CLIENT_OP			10204
@@ -621,131 +755,194 @@ void set_val (char *d, unsigned short val);
 #define MSG_CLIENT_CHANNEL_UNVOICE	10212
 #define MSG_CLIENT_CHANNEL_MUZZLE	10213
 #define MSG_CLIENT_CHANNEL_UNMUZZLE	10214
+#define MSG_CLIENT_CLASS_ADD		10250
+#define MSG_CLIENT_CLASS_DEL		10251
+#define MSG_CLIENT_CLASS_LIST		10252
+#define MSG_CLIENT_DLINE_ADD		10253
+#define MSG_CLIENT_DLINE_DEL		10254
+#define MSG_CLIENT_DLINE_LIST		10255
+#define MSG_CLIENT_ILINE_ADD		10256
+#define MSG_CLIENT_ILINE_DEL		10257
+#define MSG_CLIENT_ILINE_LIST		10258
+#define MSG_CLIENT_ELINE_ADD		10259
+#define MSG_CLIENT_ELINE_DEL		10260
+#define MSG_CLIENT_ELINE_LIST		10261
+
 #define MSG_CLIENT_SHARE_FILE		10300	/* generic media type */
 #define MSG_CLIENT_BROWSE_NEW		10301
 #define MSG_SERVER_BROWSE_RESULT_NEW	10302
 
 /* utility routines */
-int add_client (CONNECTION *);
-void add_timer (int, int, timer_cb_t, void *);
-char *append_string (char *in, const char *fmt, ...);
-int bind_interface (int, unsigned int, int);
+
+#if HAVE_POLL
+void    remove_fd (int fd);
+#endif
+
+int     acl_connection_allowed (unsigned int);
+void    acl_init (void);
+void    acl_destroy (void);
+void    acl_save (void);
+void    acl_sync (CONNECTION *);
+int     add_client (CONNECTION *, int is_server);
+void    add_fd (int fd);
+int     add_timer (int, int, timer_cb_t, void *);
+char   *append_string (char *in, const char *fmt, ...);
+void    auto_link (void);
+int     bind_interface (int, unsigned int, int);
 BUFFER *buffer_append (BUFFER *, BUFFER *);
 BUFFER *buffer_consume (BUFFER *, int);
-void buffer_free (BUFFER *);
-int buffer_size (BUFFER *);
-int buffer_decompress (BUFFER *, z_streamp, char *, int);
-int buffer_validate (BUFFER *);
-void cancel_search (CONNECTION * con);
-int check_ban (CONNECTION *, const char *, const char *);
-int check_connect_status (int);
-int check_pass (const char *info, const char *pass);
-void close_db (void);
-void complete_connect (CONNECTION * con);
-void config (const char *);
-void config_defaults (void);
+void    buffer_free (BUFFER *);
+int     buffer_size (BUFFER *);
+int     buffer_decompress (BUFFER *, z_streamp, char *, int);
+int     buffer_validate (BUFFER *);
+void    cancel_search (CONNECTION * con);
+int     check_ban (CONNECTION *, const char *, const char *);
+int     check_class (CONNECTION *, ip_info_t *);
+int     check_connect_status (int);
+int     check_pass (const char *info, const char *pass);
+void    cleanup_ip_info (void);
+void    clear_read (int);
+void    clear_write (int);
+void    close_db (void);
+void    complete_connect (CONNECTION * con);
+int     config (int);
+void    config_defaults (void);
 USERDB *create_db (USER *);
-void dump_channels(void);
-void exec_timers (time_t);
-void expand_hex (char *, int);
-void expire_bans (void);
-void fdb_garbage_collect (HASH *);
-void finalize_compress (SERVER *);
+void    destroy_connection (CONNECTION *);
+void    dump_channels (void);
+void    exec_timers (time_t);
+void    expand_hex (char *, int);
+void    expire_bans (void);
+void    expire_searches (void);
+void    expire_whowas (void);
+void    finalize_compress (SERVER *);
 CHANNEL *find_channel (LIST *, const char *);
-int form_message (char *, int, int, const char *, ...);
-void free_ban (BAN *);
-void free_channel (CHANNEL *);
-void free_config (void);
-void free_datum (DATUM *);
-void free_flist (FLIST *);
-void free_hotlist (HOTLIST *);
-void free_pointer (void *);
-void free_timers (void);
-void free_user (USER *);
-char *generate_nonce (void);
-char *generate_pass (const char *pass);
-int get_level (const char *);
+CHANUSER *find_chanuser (LIST *, USER *);
+server_auth_t *find_server_auth (const char *);
+int     form_message (char *, int, int, const char *, ...);
+void    free_ban (BAN *);
+void    free_channel (CHANNEL *);
+void    free_config (void);
+void    free_datum (DATUM *);
+void    free_pointer (void *);
+void    free_server_auth (server_auth_t *);
+void    free_timers (void);
+void    free_user (USER *);
+void    free_whowas (whowas_t *);
+char   *generate_nonce (void);
+char   *generate_pass (const char *pass);
+int     get_level (const char *);
 unsigned short get_local_port (int);
-void get_random_bytes (char *d, int);
-void handle_connection (CONNECTION *);
-void init_compress (CONNECTION *, int);
-int init_db (void);
-void init_random (void);
-int init_server (const char *);
-int invalid_channel (const char *);
-void invalid_channel_msg (CONNECTION *);
-int invalid_nick (const char *);
-void invalid_nick_msg (CONNECTION *);
-int is_chanop(CHANNEL*,USER*);
-int is_filtered (const char *);
-int is_ignoring (LIST *, const char *);
-int is_linked (CONNECTION *, const char *);
-int is_server (const char *);
-int glob_match(const char *, const char *);
-int load_bans (void);
-void load_channels (void);
-void load_filter (void);
-void log (const char *fmt, ...);
+void    get_random_bytes (char *d, int);
+int     glob_match (const char *, const char *);
+void    handle_connection (CONNECTION *);
+void    init_compress (CONNECTION *, int);
+int     init_db (void);
+void    init_random (void);
+int     init_server (void);
+int     invalid_channel (const char *);
+void    invalid_channel_msg (CONNECTION *);
+int     invalid_host (const char *);
+int     invalid_nick (const char *);
+void    invalid_nick_msg (CONNECTION *);
+void    ip_info_free (ip_info_t *);
+int     is_address (char *, unsigned int *, unsigned int *);
+int     is_blocked (char *);
+int     is_chanop (CHANNEL *, USER *);
+int     is_filtered (char *);
+int     is_ignoring (LIST *, const char *);
+int     is_linked (const char *);
+int     is_server (const char *);
+void    kill_user_internal (CONNECTION *, USER *, const char *, USER *,
+			    const char *, ...);
+void    lag_detect (void *);
+int     load_bans (void);
+void    load_block (void);
+void    load_channels (void);
+int     load_class (void);
+void    load_filter (void);
+void    load_server_auth (void);
+void    log (const char *fmt, ...);
 unsigned int lookup_ip (const char *host);
-int make_tcp_connection (const char *host, int port, unsigned int *ip);
-void motd_init(void);
-void motd_close(void);
-char *my_ntoa (unsigned int);
-USER *new_user (void);
+int     make_tcp_connection (const char *host, int port, unsigned int *ip);
+void    motd_init (void);
+void    motd_close (void);
+char   *my_ntoa (unsigned int);
 CHANNEL *new_channel (void);
 CONNECTION *new_connection (void);
-int new_tcp_socket (int);
-char *next_arg (char **);
-char *next_arg_noskip (char **);
-time_t next_timer (void);
-char *normalize_ban(char *, char *, int);
-void nosuchuser (CONNECTION *);
-void nosuchchannel (CONNECTION*);
-void notify_mods (unsigned int, const char *, ...);
-void notify_ops (CHANNEL *, const char *, ...);
-void part_channel (CHANNEL *, USER *);
-void pass_message (CONNECTION *, char *, size_t);
-void pass_message_args (CONNECTION * con, unsigned int msgtype,
-			const char *fmt, ...);
-void permission_denied (CONNECTION * con);
-int pop_user (CONNECTION * con, char **pkt, USER ** user);
-int pop_user_server (CONNECTION * con, int tag, char **pkt, char **nick, USER ** user);
-void print_args (int, char **);
-void queue_data (CONNECTION *, char *, int);
-void remove_connection (CONNECTION *);
-void remove_links (const char *);
-void remove_user (CONNECTION *);
-int safe_realloc (void **, int);
-int save_bans (void);
-void send_cmd (CONNECTION *, unsigned int msgtype, const char *fmt, ...);
-void send_cmd_pre (CONNECTION *, unsigned int msgtype, const char *pre, const char *fmt, ...);
-int send_queued_data (CONNECTION * con);
-void send_user (USER *, int, char *fmt, ...);
-int set_keepalive (int, int);
-int set_data_size (int);
-int set_max_connections (int);
-int set_nonblocking (int);
-int set_rss_size (int);
-int set_tcp_buffer_len (int, int);
-int split_line (char **template, int templatecount, char *pkt);
-char *strlower (char *);
-void synch_server (CONNECTION *);
-LIST *tokenize (char *);
-void truncate_reason (char *);
-void unparsable(CONNECTION *);
-int userdb_dump (void);
-int userdb_init (void);
-void userdb_free (USERDB *);
-int validate_user (USER *);
-int validate_channel (CHANNEL *);
-int validate_connection (CONNECTION *);
-int validate_hotlist (HOTLIST *);
+int     new_tcp_socket (int);
+char   *next_arg (char **);
+char   *next_arg_noskip (char **);
+time_t  next_timer (void);
+char   *normalize_ban (char *, char *, int);
+void    nosuchuser (CONNECTION *);
+void    nosuchchannel (CONNECTION *);
+void    notify_mods (unsigned int, const char *, ...);
+void    notify_ops (CHANNEL *, const char *, ...);
+void    part_channel (CHANNEL *, USER *);
+void    pass_message (CONNECTION *, char *, size_t);
+void    pass_message_args (CONNECTION * con, unsigned int msgtype,
+			   const char *fmt, ...);
+void    permission_denied (CONNECTION * con);
+int     pop_user (CONNECTION * con, char **pkt, USER ** user);
+int     pop_user_server (CONNECTION * con, int tag, char **pkt, char **nick,
+			 USER ** user);
+void    print_args (int, char **);
+void    queue_data (CONNECTION *, char *, int);
+void    remove_connection (CONNECTION *);
+void    remove_links (const char *);
+void    remove_user (CONNECTION *);
+int     safe_realloc (void **, int);
+int     save_bans (void);
+int     save_class (void);
+void    send_all_clients (int, const char *, ...);
+void    send_cmd (CONNECTION *, unsigned int msgtype, const char *fmt, ...);
+void    send_cmd_pre (CONNECTION *, unsigned int msgtype, const char *pre,
+		      const char *fmt, ...);
+int     send_queued_data (CONNECTION * con);
+int     send_to_channel (char *, char *, int);
+void    send_user (USER *, int, char *fmt, ...);
+int     set_keepalive (int, int);
+int     set_data_size (int);
+void    set_fd (int, int);
+int     set_max_connections (int);
+int     set_nonblocking (int);
+void    set_read (int);
+int     set_rss_size (int);
+int     set_tcp_buffer_len (int, int);
+void    set_write (int);
+int     split_line (char **template, int templatecount, char *pkt);
+void    stat_server_push (void);
+void    stat_server_read (void);
+char   *strfcpy (char *, const char *, size_t);
+char   *strlower (char *);
+void    sync_channel_bans (CONNECTION *, CHANNEL *);
+void    sync_channel_user (CONNECTION *, CHANNEL *, CHANUSER *);
+void    synch_server (CONNECTION *);
+void    timer_set_interval (unsigned int, int);
+LIST   *tokenize (char *, LIST **);
+void    truncate_reason (char *);
+void    unparsable (CONNECTION *);
+int     userdb_dump (void);
+int     userdb_init (void);
+void    userdb_free (USERDB *);
+void    logmode_change (char *, char *, int, int);
+unsigned int set_usermode (char *, unsigned int);
+void    config_user_level (char *);
 
-#define HANDLER(f) void f (CONNECTION *con, unsigned short tag, unsigned short len, char *pkt)
+#if DEBUG
+int     validate_user (USER *);
+int     validate_channel (CHANNEL *);
+int     validate_connection (CONNECTION *);
+int     validate_flist (FLIST *);
+#endif
+
+#define HANDLER(f) void f (CONNECTION *con, unsigned short tag, u_short len, char *pkt)
 /* this is not a real handler, but has the same arguments as one */
 HANDLER (dispatch_command);
 
 /* protocol handlers */
+HANDLER (add_class);
 HANDLER (add_file);
 HANDLER (add_directory);
 HANDLER (add_hotlist);
@@ -756,6 +953,8 @@ HANDLER (announce);
 HANDLER (ban);
 HANDLER (banlist);
 HANDLER (browse);
+HANDLER (browse_direct);
+HANDLER (browse_direct_ok);
 HANDLER (browse_new);
 HANDLER (change_data_port);
 HANDLER (change_email);
@@ -764,22 +963,20 @@ HANDLER (change_pass);
 HANDLER (channel_ban);
 HANDLER (channel_banlist);
 HANDLER (channel_clear_bans);
-HANDLER (channel_drop);
 HANDLER (channel_invite);
 HANDLER (channel_level);
 HANDLER (channel_limit);
 HANDLER (channel_mode);
 HANDLER (channel_muzzle);
 HANDLER (channel_op);
-HANDLER (channel_op_list);
 HANDLER (channel_unban);
-HANDLER (channel_voice);
 HANDLER (channel_wallop);
 HANDLER (check_password);
-HANDLER (check_port);
+HANDLER (class_list);
 HANDLER (clear_channel);
 HANDLER (clear_ignore);
 HANDLER (client_quit);
+HANDLER (client_version_stats);
 HANDLER (cloak);
 HANDLER (cycle_client);
 HANDLER (data_port_error);
@@ -789,8 +986,12 @@ HANDLER (download_start);
 HANDLER (emote);
 HANDLER (encapsulated);
 HANDLER (full_channel_list);
+HANDLER (generic_acl_add);
+HANDLER (generic_acl_del);
+HANDLER (generic_acl_list);
 HANDLER (global_user_list);
 HANDLER (ignore);
+HANDLER (ignore_command);
 HANDLER (ignore_list);
 HANDLER (join);
 HANDLER (kick);
@@ -801,10 +1002,12 @@ HANDLER (link_info);
 HANDLER (list_channels);
 HANDLER (list_users);
 HANDLER (login);
+HANDLER (mass_kill);
 HANDLER (muzzle);
 HANDLER (nuke);
 HANDLER (part);
 HANDLER (ping);
+HANDLER (ping_all_servers);
 HANDLER (ping_server);
 HANDLER (privmsg);
 HANDLER (public);
@@ -830,7 +1033,7 @@ HANDLER (server_error);
 HANDLER (server_links);
 HANDLER (server_login);
 HANDLER (server_login_ack);
-HANDLER (server_quit);
+HANDLER (server_pong);
 HANDLER (server_reconfig);
 HANDLER (server_stats);
 HANDLER (server_usage);
@@ -840,10 +1043,12 @@ HANDLER (show_motd);
 HANDLER (upload_ok);
 HANDLER (upload_start);
 HANDLER (upload_end);
+HANDLER (time_check);
 HANDLER (topic);
 HANDLER (unban);
 HANDLER (unignore);
 HANDLER (unnuke);
+HANDLER (unshare_all);
 HANDLER (upload_request);
 HANDLER (user_ip);
 HANDLER (user_sharing);
@@ -851,9 +1056,12 @@ HANDLER (user_speed);
 HANDLER (wallop);
 HANDLER (whois);
 HANDLER (user_mode_cmd);
+HANDLER (which_server);
+HANDLER (who_was);
+HANDLER (whois_notify);
 
-#define CHECK_USER_CLASS(f) if (con->class != CLASS_USER) { log ("%s(): not USER class", f); return; }
-#define CHECK_SERVER_CLASS(f) if(con->class != CLASS_SERVER) { log ("%s(): not SERVER class", f); return; }
+#define CHECK_USER_CLASS(f) if (con->class != CLASS_USER) { log ("%s: not USER class", f); return; }
+#define CHECK_SERVER_CLASS(f) if(con->class != CLASS_SERVER) { log ("%s: not SERVER class", f); return; }
 
 #define NONULL(p) (p!=0?p:"")
 
@@ -875,7 +1083,7 @@ typedef unsigned int socklen_t;
 /*
 ** Macros to use to aid in porting code to Win32
 */
-#ifndef WIN32
+#if !defined(WIN32) || defined(__CYGWIN__)
 #define READ read
 #define WRITE write
 #define CLOSE close
@@ -897,16 +1105,16 @@ typedef unsigned int socklen_t;
 #define N_ERRNO h_errno
 #define nlogerr(f,s) _logerr(f,s,h_errno)
 
-#define SHAREDIR "/opennap"
+#define SHAREDIR "//c/opennap"
 #define PACKAGE "opennap"
-#define VERSION "0.36"
+#define VERSION "0.40"
 
 #define strcasecmp stricmp
 #define strncasecmp strnicmp
 #define vsnprintf _vsnprintf
 #define snprintf _snprintf
-#define getuid() 0	/* just fake it */
-#define getpid() 0	/* just fake it */
+#define getuid() 0		/* just fake it */
+#define getpid() 0		/* just fake it */
 
 extern int _getopt (int, char **, char *);
 
@@ -928,9 +1136,13 @@ extern int optind;
 #define WALLOPLOG_MODE	(1<<8)
 #define CLOAKLOG_MODE	(1<<9)
 #define FLOODLOG_MODE	(1<<10)
+#define PINGLOG_MODE	(1<<11)
+#define MSGLOG_MODE	(1<<12)
+#define	WHOISLOG_MODE	(1<<13)
 
 #define LOGALL_MODE (ERROR_MODE|BANLOG_MODE|CHANGELOG_MODE|\
 	KILLLOG_MODE|LEVELLOG_MODE|SERVERLOG_MODE|MUZZLELOG_MODE|PORTLOG_MODE|\
-	WALLOPLOG_MODE|CLOAKLOG_MODE|FLOODLOG_MODE)
+	WALLOPLOG_MODE|CLOAKLOG_MODE|FLOODLOG_MODE|PINGLOG_MODE|MSGLOG_MODE|\
+	WHOISLOG_MODE)
 
 #endif /* opennap_h */
