@@ -1,8 +1,8 @@
-/* Copyright (C) 2000 drscholl@users.sourceforge.net
+/* Copyright (C) 2000-1 drscholl@users.sourceforge.net
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details.
 
-   $Id: resume.c,v 1.22 2000/08/13 22:19:55 drscholl Exp $ */
+   $Id: resume.c,v 1.25 2001/02/15 08:39:45 drscholl Exp $ */
 
 #include <stdlib.h>
 #include "opennap.h"
@@ -12,11 +12,11 @@
 HANDLER (resume)
 {
 #if RESUME
-    char *av[2];
-    FLIST *flist;
-    LIST *ptr;
-    DATUM *d;
-    int fsize;
+    char   *av[2];
+    FLIST  *flist;
+    LIST   *ptr;
+    DATUM  *d;
+    int     fsize;
 #endif /* RESUME */
 
     (void) tag;
@@ -32,9 +32,9 @@ HANDLER (resume)
     }
 
     fsize = atoi (av[1]);
-    if(fsize < 1)
+    if (fsize < 1)
     {
-	send_cmd(con,MSG_SERVER_NOSUCH,"invalid file size");
+	send_cmd (con, MSG_SERVER_NOSUCH, "invalid file size");
 	return;
     }
 
@@ -45,14 +45,13 @@ HANDLER (resume)
 	for (ptr = flist->list; ptr; ptr = ptr->next)
 	{
 	    d = (DATUM *) ptr->data;
-	    if (d->size == (size_t)fsize)
+	    if (d->size == (size_t) fsize)
 	    {
 		ASSERT (validate_user (d->user));
 		send_cmd (con, MSG_SERVER_RESUME_MATCH,
-			  "%s %u %d \"%s\" %s %d %hu",
+			  "%s %u %hu \"%s\" %s %u %hu",
 			  d->user->nick, d->user->ip, d->user->port,
-			  d->filename, d->hash, d->size,
-			  d->user->speed);
+			  d->filename, d->hash, d->size, d->user->speed);
 	    }
 	}
     }
